/*
	mac2g34.C

	Convert MACPAINT images to G34 format.

	Version 1.0

	Copyright (C) 1994, Geoff Friesen B.Sc.
	All rights reserved.
*/

#if !defined(__LARGE__)
#error Large Memory Model Expected
#endif

#include <dir.H>
#include <dos.H>
#include <stdio.H>
#include <stdlib.H>
#include <string.H>

char dac [720];
char header [640];
char image [51840u];
char line [576];

int  ReadMacLine	(char *p, FILE *fp);
int  UnpackMacFile	(char *p, FILE *fp);

int main (int argc, char **argv)
{
   FILE *fp;
   char drive [MAXDRIVE];
   char dir [MAXDIR];
   char file [MAXFILE];
   char ext [MAXEXT];
   char path [MAXPATH];
   unsigned bit, byte, index;
   unsigned nx = 320, ny = 400, pos, row, x = 0, y = 0;

   fprintf (stderr, "ͻ\n");
   fprintf (stderr, " MAC2G34: Convert MACPAINT images to G34 \n");
   fprintf (stderr, "                                         \n");
   fprintf (stderr, " Version 1.0                             \n");
   fprintf (stderr, "                                         \n");
   fprintf (stderr, " Copyright (C) 1994, Geoff Friesen B.Sc. \n");
   fprintf (stderr, " All rights reserved.                    \n");
   fprintf (stderr, "ͼ\n\n");

   if (argc != 2 && argc != 6)
   {
       fprintf (stderr, "mac2g34: invalid number of arguments\n");
       fprintf (stderr, "syntax : mac2g34 filespec [x y nx ny]\n");
       return 1;
   }

   if (argc == 6)
   {
       x = atoi (argv [2]);
       y = atoi (argv [3]);
       nx = atoi (argv [4]);
       ny = atoi (argv [5]);
   }

   if (nx > 320 || ny > 400 || x+nx > 576 || y+ny > 720)
   {
       fprintf (stderr, "mac2g34: argument(s) out of range\n");
       return 1;
   }

   if ((fp = fopen (argv [1], "rb")) == NULL)
   {
       fprintf (stderr, "mac2g34: unable to open %s\n", argv [1]);
       return 1;
   }

   if (fread (header, 1, 640, fp) != 640)
   {
       fprintf (stderr, "mac2g34: unable to read MAC header\n");
       (void) fclose (fp);
       return 1;
   }

   if (memcmp (header+0x0041, "PNTG", 4))
   {
       fprintf (stderr, "mac2g34: %s is not a MacPaint file\n", argv [1]);
       (void) fclose (fp);
       return 1;
   }

   if (UnpackMacFile (image, fp) != 72)
   {
       fprintf (stderr, "mac2g34: unable to unpack image\n");
       (void) fclose (fp);
       return 1;
   }

   (void) fclose (fp);

   strcpy (path, argv [1]);
   fnsplit (path, drive, dir, file, ext);
   strcpy (ext, ".G34");
   fnmerge (path, drive, dir, file, ext);

   if ((fp = fopen (path, "wb")) == NULL)
   {
       fprintf (stderr, "mac2g34: unable to create %s\n", path);
       return 1;
   }

   memset (header, 0, 128);

   header [0] = 'G';
   header [1] = '3';
   header [2] = '4';

   *(unsigned *) (header+4) = nx;
   *(unsigned *) (header+6) = ny;

   if (fwrite (header, 1, 128, fp) != 128)
   {
       fprintf (stderr, "mac2g34: unable to write to %s\n", path);
       (void) fclose (fp);
       (void) unlink (path);
       return 1;
   }

   _ES = FP_SEG(dac);
   _DX = FP_OFF(dac);

   asm mov ax, 1017h
   asm mov bx, 0
   asm mov cx, 256
   asm int 10h

   if (fwrite (dac, 1, 768, fp) != 768)
   {
       fprintf (stderr, "mac2g34: unable to write to %s\n", path);
       (void) fclose (fp);
       (void) unlink (path);
       return 1;
   }

   fprintf (stderr, "mac2g34: converting %s to %s\n", argv [1], path);

   index = y*72;

   for (row = y; row < y+ny; row++)
   {
	pos = 0;
	for (byte = 0; byte < 72; byte++, index++)
	     for (bit = 128; bit; bit >>= 1)
		  line [pos++] = (image [index] & bit) ? 0 : 15;

	if (fwrite (line+x, 1, nx, fp) != nx)
	{
	    fprintf (stderr, "mac2g34: unable to write to %s\n", path);
	    (void) fclose (fp);
	    (void) unlink (path);
	    return 1;
	}
   }

   (void) fclose (fp);

   fprintf (stderr, "mac2g34: %s successfully generated\n", path);

   return 0;
}

int ReadMacLine (char *p, FILE *fp)
{
   int c, i, n = 0;

   do
   {
	c = fgetc (fp) & 255;
	if (c & 0x80)
	{
	    i = ((~c) & 255)+2;
	    c = fgetc (fp);
	    while (i--)
	       p [n++] = c;
	}
	else
	{
	    i = (c & 255)+1;
	    while (i--)
	       p [n++] = fgetc (fp);
	}
   }
   while (n < 72);

   if (c == EOF)
       n = 0;

   return n;
}

int UnpackMacFile (char *p, FILE *fp)
{
   int i, n;

   for (i = 0; i < 720; i++)
   {
	if ((n = ReadMacLine (p, fp)) != 72)
	    break;
	p += 72;
   }

   return n;
}